import { debounce } from 'lodash';
import { useMemo, useState, useCallback } from 'react';
import { t } from '../../utils/i18n.js';
import { itemFilter } from './filter.js';
import { useLatestAsyncCall, StaleResultError } from './useLatestAsyncCall.js';

const asyncNoop = () => Promise.resolve([]);
function useOptions(rawOptions, createCustomValue) {
  const isAsync = typeof rawOptions === "function";
  const loadOptions = useLatestAsyncCall(isAsync ? rawOptions : asyncNoop);
  const debouncedLoadOptions = useMemo(
    () => debounce((searchTerm) => {
      return loadOptions(searchTerm).then((options) => {
        setAsyncOptions(options);
        setAsyncLoading(false);
        setAsyncError(false);
      }).catch((error) => {
        if (!(error instanceof StaleResultError)) {
          setAsyncError(true);
          setAsyncLoading(false);
          if (error) {
            console.error("Error loading async options for Combobox", error);
          }
        }
      });
    }, 200),
    [loadOptions]
  );
  const [asyncOptions, setAsyncOptions] = useState([]);
  const [asyncLoading, setAsyncLoading] = useState(false);
  const [asyncError, setAsyncError] = useState(false);
  const [userTypedSearch, setUserTypedSearch] = useState("");
  const addCustomValue = useCallback(
    (opts) => {
      let currentOptions = opts;
      if (createCustomValue && userTypedSearch) {
        const customValueExists = opts.some((opt) => opt.value === userTypedSearch);
        if (!customValueExists) {
          currentOptions = [
            {
              label: userTypedSearch,
              value: userTypedSearch,
              description: t("combobox.custom-value.description", "Use custom value")
            },
            ...currentOptions
          ];
        }
      }
      return currentOptions;
    },
    [createCustomValue, userTypedSearch]
  );
  const updateOptions = useCallback(
    (inputValue) => {
      if (!isAsync) {
        setUserTypedSearch(inputValue);
        return;
      }
      setAsyncLoading(true);
      debouncedLoadOptions(inputValue);
    },
    [debouncedLoadOptions, isAsync]
  );
  const finalOptions = useMemo(() => {
    let currentOptions = [];
    if (isAsync) {
      currentOptions = addCustomValue(asyncOptions);
    } else {
      currentOptions = addCustomValue(rawOptions.filter(itemFilter(userTypedSearch)));
    }
    return currentOptions;
  }, [isAsync, addCustomValue, asyncOptions, rawOptions, userTypedSearch]);
  return { options: finalOptions, updateOptions, asyncLoading, asyncError };
}

export { useOptions };
//# sourceMappingURL=useOptions.js.map
